@@ -63,12 +63,14 @@ extern gboolean multisync_debug;
 void obex_event(obex_t *handle, obex_object_t *object, gint mode, gint event, gint obex_cmd, gint obex_rsp)
 {
   obexdata_t *userdata;
+  osync_trace(TRACE_INTERNAL, "obex event: %i", event);
 
   userdata = (obexdata_t*) OBEX_GetUserData(handle);
   switch (event)	{
   case OBEX_EV_PROGRESS:
     break;
   case OBEX_EV_REQDONE:
+    userdata->busy = 0;
     if(mode == OBEX_CLIENT) {
       client_done(handle, object, obex_cmd, obex_rsp);
     } else  {
@@ -96,7 +98,7 @@ void obex_event(obex_t *handle, obex_object_t *object, gint mode, gint event, gi
   case OBEX_EV_PARSEERR:
   case OBEX_EV_ABORT:
     userdata->state = IRMC_OBEX_REQFAILED;
-    osync_error_set(userdata->error, OSYNC_ERROR_NO_CONNECTION, NULL);
+    osync_error_set(userdata->error, OSYNC_ERROR_NO_CONNECTION, "Request failed." );
     break;
   default:
     g_print("Unknown event!\n");
@@ -151,7 +153,7 @@ void put_client_done(obex_t *handle, obex_object_t *object, gint obex_rsp) {
     return;
   }
 
-  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))	{
+  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, (uint32_t *) &hlen))	{
     if (hi == OBEX_HDR_APPARAM) {
       body = (char*) hv.bs;
       body_len = (int) hlen;
@@ -182,7 +184,7 @@ void get_client_done(obex_t *handle, obex_object_t *object, gint obex_rsp) {
     return;
   }
   
-  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))	{
+  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, (uint32_t *) &hlen))	{
     if(hi == OBEX_HDR_BODY)	{
       body = (char*) hv.bs;
       body_len = (int) hlen;
@@ -435,7 +437,7 @@ gint obex_cable_handleinput(obex_t *handle, gpointer ud, gint timeout) {
   while (userdata->state >= 0 &&
 	 (ret=select(n, &readfds, NULL, NULL, &to)) > 0) {
     if((tot = read(userdata->fd, buf, 2048)) > 0) {
-      OBEX_CustomDataFeed(handle, buf, tot);
+      OBEX_CustomDataFeed(handle, (uint8_t *) buf, tot);
     } else { 
       userdata->state = IRMC_OBEX_REQFAILED;
       osync_error_set(userdata->error, OSYNC_ERROR_NO_CONNECTION, NULL);
@@ -465,16 +467,16 @@ obex_t* irmc_obex_client(irmc_config *config) {
 
   if (bluetoothplugin) {
     g_module_symbol(bluetoothplugin,"obex_connect",
-		    (gpointer*) &bttrans.connect);
+		    (gpointer) &bttrans.connect);
     g_module_symbol(bluetoothplugin,"obex_disconnect",
-		    (gpointer*) &bttrans.disconnect);
-    g_module_symbol(bluetoothplugin,"obex_write",(gpointer*) &bttrans.write);
-    g_module_symbol(bluetoothplugin,"obex_listen",(gpointer*) &bttrans.listen);
+		    (gpointer) &bttrans.disconnect);
+    g_module_symbol(bluetoothplugin,"obex_write",(gpointer) &bttrans.write);
+    g_module_symbol(bluetoothplugin,"obex_listen",(gpointer) &bttrans.listen);
     g_module_symbol(bluetoothplugin,"obex_handleinput",
-		    (gpointer*) &bttrans.handleinput);
+		    (gpointer) &bttrans.handleinput);
   }
   userdata = (obexdata_t*) g_malloc0(sizeof(obexdata_t));
-#if OBEX_CUSTOMDATA
+#if 1 //OBEX_CUSTOMDATA
   bttrans.customdata = userdata;
 #if HAVE_IRDA
   irdatrans.customdata = userdata;
@@ -499,6 +501,7 @@ obex_t* irmc_obex_client(irmc_config *config) {
   userdata->connectmedium = config->connectmedium;
   userdata->state = IRMC_OBEX_OFFLINE;
   userdata->connected = 0;
+  userdata->busy = 0;
 
   switch(userdata->connectmedium) {
   case MEDIUM_BLUETOOTH:
@@ -563,10 +566,11 @@ gboolean irmc_obex_connect(obex_t* handle, char* target, OSyncError **error) {
   userdata->connected = 1;
   if ((object = OBEX_ObjectNew(handle, OBEX_CMD_CONNECT))) {
     if (target) {
-      hd.bs = target; 
+      hd.bs = (const uint8_t *) target; 
       OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TARGET, hd, 
 			   strlen(target), 0); 
     }
+    userdata->busy = 1;
     ret = OBEX_Request(handle, object);
     if (ret < 0) {
       osync_error_set(error, OSYNC_ERROR_GENERIC, "Cannot connect via OBEX.");
@@ -574,7 +578,9 @@ gboolean irmc_obex_connect(obex_t* handle, char* target, OSyncError **error) {
     }
   }
   userdata->state = IRMC_OBEX_CONNECTING;
-  irmc_obex_handleinput(handle, 10);
+  while ( userdata->busy ) {
+    irmc_obex_handleinput(handle, 10);
+  }
   if (userdata->state == IRMC_OBEX_REQDONE) {
     if (strlen(userdata->irunit.serial) > 0) {
 #if HAVE_IRDA
@@ -608,13 +614,16 @@ gboolean irmc_obex_disconnect(obex_t* handle, OSyncError **error) {
   userdata = (obexdata_t*) OBEX_GetUserData(handle);  
   if (userdata->connected) {
     if ((object = OBEX_ObjectNew(handle, OBEX_CMD_DISCONNECT))) {
+      userdata->busy = 1;
       if(OBEX_Request(handle, object) < 0)  {
         osync_error_set(error, OSYNC_ERROR_GENERIC, "Cannot disconnect from OBEX.");
         return FALSE;
       }
     }
     userdata->state = IRMC_OBEX_DISCONNECTING;
-    irmc_obex_handleinput(handle, 10);
+    while ( userdata->busy ) {
+      irmc_obex_handleinput(handle, 10);
+    }
     OBEX_TransportDisconnect(handle);
     userdata->connected = 0;
   }
@@ -664,6 +673,11 @@ char* irmc_obex_get_serial(obex_t* handle) {
 gboolean irmc_obex_put(obex_t* handle, char* name, char *type,
 		 char *body, gint body_size, char *rspbuf, int *rspbuflen,
 		 char *apparam, int apparamlen, OSyncError **error) {
+  
+  osync_trace(TRACE_ENTRY, "%s(%p, %s, %p, %p, %i, %p, %p, %s, %i, %p)",
+	__func__, handle, name, type, body, body_size, rspbuf, rspbuflen,
+	apparam, apparamlen, error);
+
   obex_object_t *object; 
   obex_headerdata_t hd; 
   char unicodename[1024];
@@ -675,49 +689,62 @@ gboolean irmc_obex_put(obex_t* handle, char* name, char *type,
     hd.bq4 = body_size; 
     OBEX_ObjectAddHeader(handle, object, OBEX_HDR_LENGTH, hd, 4, 0); 
     /* Add unicode name header*/ 
-    namesize = OBEX_CharToUnicode(unicodename, name, 1024);
-    hd.bs = unicodename; 
+    namesize = OBEX_CharToUnicode((uint8_t *) unicodename, (const uint8_t *) name, 1024);
+    hd.bs = (const uint8_t *) unicodename; 
     OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME, hd, 
 			 namesize, 0); 
     if (type) {
-      hd.bs = type;
+      hd.bs = (const uint8_t *) type;
       OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TYPE, hd, 
 			 strlen(type), 0); 
     }
     if (apparam) {
-      hd.bs = apparam;
+      hd.bs = (const uint8_t *) apparam;
       OBEX_ObjectAddHeader(handle, object, OBEX_HDR_APPARAM, hd, 
 			   apparamlen, 0); 
     }
     if (body) {
       /* Add body header*/ 
-      hd.bs = body; 
+      hd.bs = (const uint8_t *) body;
+      osync_trace(TRACE_SENSITIVE, "OBEX-OUT:\n%s\n", hd.bs); 
       OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY, hd, body_size, 0); 
     }
+    userdata->busy = 1;
     if(OBEX_Request(handle, object) < 0) {
       osync_error_set(error, OSYNC_ERROR_GENERIC, "Cannot put a item on mobile device");
+      osync_trace(TRACE_EXIT, "%s: FALSE", __func__);
       return FALSE;
     }
     userdata->state = IRMC_OBEX_PUTTING;
     userdata->databuf = rspbuf;
     userdata->databuflen = rspbuflen;
-    irmc_obex_handleinput(handle, 30);
-    if (userdata->state == IRMC_OBEX_REQDONE)
+    while ( userdata->busy ) {
+      irmc_obex_handleinput(handle, 30);
+    }
+    if (userdata->state == IRMC_OBEX_REQDONE) {
+      osync_trace(TRACE_EXIT, "%s: FALSE", __func__);
       return TRUE;
+    }
 
     if ( userdata->error ) {
       error = userdata->error;
+      osync_trace(TRACE_EXIT, "%s: FALSE", __func__);
       return FALSE;
-    } else
+    } else {
+      osync_trace(TRACE_EXIT, "%s: TRUE", __func__);
       return TRUE;
+    }
   }
 
   osync_error_set(error, OSYNC_ERROR_GENERIC, "Request failed");
+  osync_trace(TRACE_EXIT, "%s: FALSE", __func__);
   return FALSE;
 }
 
 gboolean irmc_obex_get(obex_t *handle, char* name, char* buffer, int *buflen, OSyncError **error)
 {
+  osync_trace(TRACE_ENTRY, "%s(%p, %s, %p, %p, %p)", __func__, handle, name, buffer, buflen, error);
+
   obex_object_t *object;
   obex_headerdata_t hd;
   char unicodename[1024];
@@ -727,27 +754,35 @@ gboolean irmc_obex_get(obex_t *handle, char* name, char* buffer, int *buflen, OS
   userdata = (obexdata_t*) OBEX_GetUserData(handle);  
   if((object = OBEX_ObjectNew(handle, OBEX_CMD_GET))) {
     /* Add unicode name header*/ 
-    namesize = OBEX_CharToUnicode(unicodename, name, 1024);
-    hd.bs = unicodename; 
+    namesize = OBEX_CharToUnicode((uint8_t *) unicodename, (const uint8_t *) name, 1024);
+    hd.bs = (const uint8_t *) unicodename; 
     OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME, hd, 
 			 namesize, 0); 
     userdata->databuf = buffer;
     userdata->databuflen = buflen;
+    userdata->busy = 1;
+    //FIXME - ignore errors here - 09/14/2006 cstender
+    //userdata->error = error;
     OBEX_Request(handle, object);
     userdata->state = IRMC_OBEX_GETTING;
-    irmc_obex_handleinput(handle, 30);
+    while ( userdata->busy ) {
+      irmc_obex_handleinput(handle, 30);
+    }
     if (userdata->state == IRMC_OBEX_REQDONE) {
+      osync_trace(TRACE_EXIT, "%s : TRUE", __func__);
       return TRUE;
     }
 
     if (userdata->error) {
       error = userdata->error;
-      return FALSE;
+      goto error;
     } else
+      osync_trace(TRACE_EXIT, "%s : TRUE", __func__);
       return TRUE;
   }
-
+error:
   osync_error_set(error, OSYNC_ERROR_GENERIC, "Request failed");
+  osync_trace(TRACE_EXIT, "%s: FALSE", __func__);
   return FALSE;
 }
 
